<?php
global $LANGUAGE;
global $IMPORT_OTHERWIKI_ENCODE_OPTIONS;
global $IMPORT_OTHERWIKI_ENCODE;
global $IMPORT_OTHERWIKI_IGNORE_PAGES;

define('IMPORT_OTHERWIKI_PKWK_SPLITTER', '>>>>>>>>>>');
define('IMPORT_OTHERWIKI_LOCALZONE', date('Z'));

$LANGUAGE['ja']['PukiWiki install directory'] = 'PukiWikiをインストールしたディレクトリ';
$LANGUAGE['ja']['PukiWiki data file not found.'] = 'PukiWikiのデータファイルが見つかりませんでした．';
$LANGUAGE['ja']['PukiWiki encoding'] = 'PukiWikiの文字コード';
$LANGUAGE['ja']['Recently updated pages'] = '最近更新されたページ';

$IMPORT_OTHERWIKI_IGNORE_PAGES = array(
    '|^PukiWiki$|',
    '|^PukiWiki/.*|',
    '|^:config.*|',
    '|^:RenameLog$|',
    '|^FormatRule$|',
    '|^FormattingRules$|',
    '|^整形ルール$|',
    '|^Help$|', '|^ヘルプ$|',
    '|^RecentChanges$|', '|^RecentDeleted$|',
    );

$IMPORT_OTHERWIKI_ENCODE = 'EUC-JP';
$IMPORT_OTHERWIKI_ENCODE_OPTIONS = array('EUC-JP' => 'EUC-JP', 'UTF-8' => 'UTF-8');

function import_otherwiki_form($form) {
    global $IMPORT_OTHERWIKI_ENCODE_OPTIONS;
    $dl = dom_append_child($form, 'dl');

    $dt = dom_append_child($dl, 'dt');
    dom_append_text($dt, l('PukiWiki install directory'));
    $dd = dom_append_child($dl, 'dd');
    dom_append_child($dd, 'input', '',
		     array('type' => 'text', 'name' => 'otherwiki_path', 'required' => 'required'));

    $dt = dom_append_child($dl, 'dt');
    dom_append_text($dt, l('PukiWiki encoding'));
    $dd = dom_append_child($dl, 'dd');
    dom_append_selector($dd, 'otherwiki_encode', $IMPORT_OTHERWIKI_ENCODE_OPTIONS);
}

function import_otherwiki_form_check($args) {
    global $IMPORT_OTHERWIKI_ENCODE_OPTIONS;
    if(!isset($args['otherwiki_path']) || $args['otherwiki_path'] === '' ||
       !isset($args['otherwiki_encode']) ||
       !isset($IMPORT_OTHERWIKI_ENCODE_OPTIONS[$args['otherwiki_encode']])) {
	message_error(l('Invalid parameter.'));
	return false;
    }
    if(!is_file($args['otherwiki_path'] . '/pukiwiki.ini.php') ||
       !is_dir($args['otherwiki_path'] . '/wiki') ||
       !is_dir($args['otherwiki_path'] . '/backup')) {
	message_error(l('PukiWiki data file not found.'));
	return false;
    }
    return true;
}

function import_otherwiki_create_queue($args) {
    global $IMPORT_OTHERWIKI_ENCODE;
    $IMPORT_OTHERWIKI_ENCODE = $args['otherwiki_encode'];

    $queue = array();
    $pukiwiki_dir = $args['otherwiki_path'];
    $pukiwiki_backup_dir = $pukiwiki_dir . '/backup';
    $pukiwiki_page_dir = $pukiwiki_dir . '/wiki';
    $pukiwiki_attach_dir = $pukiwiki_dir . '/attach';

    $pukiwiki_backup_filenames = scandir($pukiwiki_backup_dir);
    foreach($pukiwiki_backup_filenames as $pukiwiki_filename) {
	if(preg_match('/^([0-9a-f]+)\.gz$/i', $pukiwiki_filename, $matches)) {
	    $pagename = import_otherwiki_convert_encoding(ascii_decode($matches[1]));
	    $pagename = page_path_to_pagename($pagename);
	    $queue[$pagename]['pagename'] = $pagename;
	    $queue[$pagename]['wiki_backup_file'] = $pukiwiki_filename;
	}
    }

    $pukiwiki_filenames = scandir($pukiwiki_page_dir);
    foreach($pukiwiki_filenames as $pukiwiki_filename) {
	if(preg_match('/^([0-9a-f]+).txt$/i', $pukiwiki_filename, $matches)) {
	    $pagename = import_otherwiki_convert_encoding(ascii_decode($matches[1]));
	    $pagename = page_path_to_pagename($pagename);
	    $queue[$pagename]['pagename'] = $pagename;
	    $queue[$pagename]['wiki_file'] = $pukiwiki_filename;
	}
    }

    $pukiwiki_attach_filenames = scandir($pukiwiki_attach_dir);
    foreach($pukiwiki_attach_filenames as $pukiwiki_filename) {
	if(preg_match('/^([0-9a-f]+)_([0-9a-f]+)(:?\.(\d+))?$/i', $pukiwiki_filename, $matches)) {
	    $pagename = import_otherwiki_convert_encoding(ascii_decode($matches[1]));
	    $filename = import_otherwiki_convert_encoding(ascii_decode($matches[2]));
	    $pagename = $pagename . '/' . $filename;
	    $pagename = page_path_to_pagename($pagename);
	    $queue[$pagename]['pagename'] = $pagename;
	    if(isset($matches[3]))
		$queue[$pagename]['attach_backup_files'][] = $pukiwiki_filename;
	    else
		$queue[$pagename]['attach_file'] = $pukiwiki_filename;
	}
    }
    $queue = array_values($queue);

    return $queue;
}

function import_otherwiki_processor($args, $dom, &$func_arg, &$job) {
    global $IMPORT_OTHERWIKI_ENCODE;
    global $IMPORT_OTHERWIKI_CURRENT_PAGENAME;

    $IMPORT_OTHERWIKI_ENCODE = $args['otherwiki_encode'];
    
    $pagename = $job['pagename'];
    $IMPORT_OTHERWIKI_CURRENT_PAGENAME = $pagename;

    if(import_otherwiki_is_ignore($pagename))
	return;

    $page = page_read($pagename);
    page_truncate($page);
    $page = page_read($pagename);

    $btime = null;
    if(isset($job['wiki_backup_file'])) {
	$btime = pukiwiki_make_backups($page, $args['otherwiki_path'] . '/backup/' . $job['wiki_backup_file']);
    }
    
    if(isset($job['wiki_file'])) {
	$pukiwiki_file = $args['otherwiki_path'] . '/wiki/' . $job['wiki_file'];
	$mtime = filemtime($pukiwiki_file);
	$source = file_get_contents($pukiwiki_file);
	$source = import_otherwiki_convert_encoding($source);
	$page = page_read($pagename);
	pukiwiki_make_backup($page, $args['start_time'], $source);
	$contents = pukiwiki_convert($page, $source);
	$page['force_mtime'] = $mtime;
	if(!is_null($btime))
	    $page['meta']['btime'] = $btime;
	page_write($page, $contents);
    }

    if(isset($job['attach_backup_files'])) {
	foreach($job['attach_backup_files'] as $attach_backup_file) {
	    $pukiwiki_file = $args['otherwiki_path'] . '/attach/' . $attach_backup_file;
	    $backup_time = filemtime($pukiwiki_file);
	    if(!is_null($btime) || $backup_time < $btime)
		$btime = $backup_time;
	    pukiwiki_make_attach($page, $pukiwiki_file, $backup_time);
	}
    }

    if(isset($job['attach_file'])) {
	$pukiwiki_file = $args['otherwiki_path'] . '/attach/' . $job['attach_file'];
	if(!is_null($btime))
	    $page['meta']['btime'] = $btime;
	pukiwiki_make_attach($page, $pukiwiki_file);
    }

}

function import_otherwiki_is_ignore($pagename) {
    global $IMPORT_OTHERWIKI_IGNORE_PAGES;
    foreach($IMPORT_OTHERWIKI_IGNORE_PAGES as $pat) {
	if(preg_match($pat, $pagename))
	    return true;
    }
    return false;
}
function import_otherwiki_convert_encoding($source) {
    global $IMPORT_OTHERWIKI_ENCODE;
    return mb_convert_encoding($source, 'UTF-8', $IMPORT_OTHERWIKI_ENCODE);
}

function pukiwiki_convert(&$page, $source) {
    $page['meta']['type'] = 'wiki';

    $output = array();
    
    $lines = explode("\n", $source);
    $lines[] = '';
    $table_format = null;
    $list_start_line_no = null;
    $line_no = 0;
    foreach($lines as $line) {
	$is_list_start = false;
	$is_list_finish = false;
	if(preg_match('/^(\*+)(.*?)\s*\[\#.*\]\s*$/', $line, $matches)) {
	    $output[] = $matches[1] . pukiwiki_convet_inline($matches[2]);
	    $is_list_finish = true;
	} else if(preg_match('/^(\*+)(.*)$/', $line, $matches)) {
	    $output[] = $matches[1] . pukiwiki_convet_inline($matches[2]);
	    $is_list_finish = true;
	} else if(preg_match('/^ (.*)$/', $line, $matches)) {
	    $output[] = ' ' . $matches[1];
	} else if(preg_match('/^\|(.*)\|\s*c\s*$/', $line, $matches)) {
	    $fields = explode('|', $matches[1]);
	    $format = '';	    
	    foreach($fields as $field) {
		$field = strtoupper(trim($field));
		switch($field) {
		case 'LEFT:': $format .= 'l'; break;
		case 'RIGHT:': $format .= 'r'; break;
		case 'CENTER:': $format .= 'c'; break;
		default: $format .= 'n'; break;
		}
	    }
	    $table_format = $format;
	} else if(preg_match('/^\|(.*)$/', $line, $matches)) {
	    $output_line = pukiwiki_convet_inline($line);
	    if(!is_null($table_format)) {
		$output_line .= $table_format;
		$table_format = null;
	    }
	    $output[] = $output_line;
	} else if(preg_match('/^\-{4,}\s*$/', $line, $matches)) {
	    $output[] = '====';
	} else if(preg_match('/^([-+]{1,3})(.*)$/', $line, $matches)) {
	    $output[] = $matches[1] . pukiwiki_convet_inline($matches[2]);
	    $is_list_start = true;
	    $is_list_finish = true;
	} else if(preg_match('/^#(\w+)(?:\((.*?)\))?/', $line, $matches)) {
	    $output[] = pukiwiki_convet_plugin($matches, 'block');
	} else if($line === ''){
	    $output[] = '';
	    $is_list_finish = true;
	} else {
	    $output[] = pukiwiki_convet_inline($line);
	}

	if($is_list_finish) {
	    if(!is_null($list_start_line_no) && $line_no - 1 > $list_start_line_no) {
		if(preg_match('/^([-+]{1,3})(.*)/', $output[$list_start_line_no], $matches)) {
		    $output[$list_start_line_no] =  $matches[1] . '{' . $matches[2];
		    if(!isset($output[$line_no - 1])) {
			print_r($output);
			debug_log($page['name'] . ' line:' . $line);
		    }
		    $output[$line_no - 1] .= "\n}";
		}
	    }
	    $list_start_line_no = null;
	}
	if($is_list_start)
	    $list_start_line_no = $line_no;
	
	$line_no++;
    }
    array_pop($output);
    
    return implode("\n", $output);
}

function pukiwiki_convet_inline($source) {
    $source = preg_replace_callback('/\&(\w+)(?:\((.*?)\))?(?:\{(.*)\})?;/', 'pukiwiki_convet_plugin', $source);
    return $source;
}

function pukiwiki_convet_plugin($matches, $type = 'inline') {
    global $IMPORT_OTHERWIKI_CURRENT_PAGENAME;
    //global $plugins;

    $name = strtolower($matches[1]);
    if(isset($matches[2]))
	$args = explode(',', $matches[2]);
    else
	$args = array();
    $target = default_value($matches[3]);

    //$plugins[$name]['uses'][] = sprintf('[%s] %s', $current_pagename, $matches[0]);

    switch($name) {
    case 'ref':
    case 'media':
	$pagename = array_shift($args);
	if(strpos($pagename, '/') === false)
	    $pagename = './' . $pagename;
	if(pukiwiki_arg_search($args, 'noimg')) {
	    return '[[' . $pagename . ']]';
	} else {
	    $new_args = array('[['. $pagename .']]');
	    foreach($args as $arg) {
		if(preg_match('/^\d+%$/', trim($arg)))
		    $new_args[] = $arg;
	    }
	    return '&include(' . implode(',', $new_args) . ');';
	}

    case 'color':
    case 'aname':
    case 'p':
    case 'paint':
    case 'amp':
    case 'heart':
    case 'smile':
    case 'bigsmile':
    case 'huh':
    case 'oh':
    case 'wink':
    case 'sad':
    case 'worried':
	return $matches[0];

    case 'sum':
	return '={sum(up)}';

    case 'math':
	return '&math{' . implode(',', $args) . '}';

    case 'include':
	return '&include([[' . $args[0] . ']]);';

    case 'recent':
	return '*'.l('Recently updated pages')."\n".'&recent('. implode(',', $args) .');';

    case 'new':
	$target = preg_replace('/\(.*\) */', '', $target);
	return '&time(new){' . $target . '}';
	
    case 'di':
	if($type == 'inline') {
	    return pukiwiki_convet_inline($target);
	}
	$args[] = 'noname';
	$args[] = 'nodate';
    case 'comment':
	$new_args = array();
	if(!pukiwiki_arg_search($args, 'noname'))
	    $new_args[] = 'name';
	if(!pukiwiki_arg_search($args, 'nodate'))
	    $new_args[] = 'date';
	if(pukiwiki_arg_search($args, 'below'))
	    $new_args[] = 'below';
	if(pukiwiki_arg_search($args, 'above'))
	    $new_args[] = 'above';
	
	if(!empty($new_args[0]) && $new_args[0] == 'name' &&
	   !empty($new_args[1]) && $new_args[1] == 'date') {
	    array_shift($new_args);
	    array_shift($new_args);
	    return '&comment(' . implode(',', $new_args) . ');';
	} else {
	    return '&listadd(' . implode(',', $new_args) . ');';
	}

    case 'cal':
    case 'calendar':
    case 'terus_cal':
	$new_args = array();
	if(!empty($args[0]))
	    $new_args[] = '[[' . $args[0] . ']]';
	return '&calendar(' . implode(',', $new_args) . ');';

    case 'calendar_viewer':
	$pagename = pukiwiki_trim_pagename(default_value($args[0]));
	$new_args = array('[[' . $pagename . ']]');
	if(pukiwiki_arg_search($args, 'past'))
	    $new_args[] = 'past';
	if(pukiwiki_arg_search($args, 'future'))
	    $new_args[] = 'future';
	return '&datepages(' . implode(',', $new_args) . ');';

    case 'contents':
	return '&index;';

    case 'ls':
	return '&pages;';
    case 'ls2':
	$new_args = array();
	$pagename = pukiwiki_trim_pagename(default_value($args[0]));
	if($pagename !== '')
	    $new_args[] = '[[' . $pagename . ']]';
	return '&pages(' . implode(',', $new_args) . ');';
    case 'norelated':
	return '';
    }
    debug_log('unkown pukiwiki plugin [[' . $IMPORT_OTHERWIKI_CURRENT_PAGENAME . ']] ' . $matches[0]);
    
    return $matches[0];
}

function pukiwiki_arg_search(&$args, $needle) {
    foreach($args as $arg) {
	if(trim($arg) == $needle)
	    return true;
    }
    return false;
}

function pukiwiki_make_backups(&$page, $filepath) {
    $min_backup_time = null;

    $fp = gzopen($filepath, 'r');
    $buf = '';
    $backup_time = 0;
    while(($line = fgets($fp)) !== false) {
	$line = import_otherwiki_convert_encoding($line);
	if(preg_match('/^'. preg_quote(IMPORT_OTHERWIKI_PKWK_SPLITTER, '/') . '\s*(\d+)/', $line, $matches)) {
	    if(!empty($buf) && !empty($backup_time)) {
		pukiwiki_make_backup($page, $backup_time, $buf);
		$buf = '';
	    }
	    $backup_time = $matches[1];
	    $backup_time += IMPORT_OTHERWIKI_LOCALZONE;
	    if(empty($min_backup_time) || $backup_time < $min_backup_time)
		$min_backup_time = $backup_time;
	} else {
	    $buf .= $line;
	}
    }

    if(!empty($buf) && !empty($backup_time)) {
	pukiwiki_make_backup($page, $backup_time, $buf);
	$buf = '';
    }

    gzclose($fp);
    return $min_backup_time;
}

function pukiwiki_make_backup(&$page, $backup_time, &$contents) {
    return storage_page_write_backup($page, $backup_time, $contents, $error);
}

function pukiwiki_make_attach(&$page, $pukiwiki_file, $backup_time = 0) {
    global $pages;

    $mtime = filemtime($pukiwiki_file);
    $page['force_mtime'] = $mtime;
    $page['meta']['type'] = 'file';
    $page['meta']['original_filename'] = page_get_simplename($page['name']);

    $sizeinfo = @getimagesize($pukiwiki_file);
    if($sizeinfo !== false) {
	$page['meta']['Content-type'] = $sizeinfo['mime'];
    }

    $contents = array('upload_file' => $pukiwiki_file);
    $content_type_handler = &file_get_content_type_handler($page);
    if(!empty($content_type_handler['write']) && function_exists($content_type_handler['write'])) {
	$content_type_handler['write']($page, $contents);
    }
    if($backup_time == 0)
	page_write($page, $contents);
    else
	storage_page_write_backup($page, $backup_time, $contents, $error);
}

function pukiwiki_trim_pagename($pagename) {
    return trim($pagename, '[]');
}

/*
  include(APP_DIR_PATH.'/otherwiki/pukiwiki.inc');
  import_otherwiki_test();
  exit(1);
*/
function import_otherwiki_test() {
    header('Content-type: text/plain');
    $source = file_get_contents('test.txt');
    $page = page_read('Top');
    $output = pukiwiki_convert($page, $source);
    
    print($output);
    print("-----------------------------------------\n");
    print($source);
}
?>
